<?php
function parsepng($file)
{
	// Extract info from a PNG file
	$f = fopen($file,'rb');
	if(!$f)
		echo('Can\'t open image file: '.$file);
	$info = parsepngstream($f,$file);
	fclose($f);
	return $info;
}

function parsepngstream($f, $file)
{
	// Check signature
	if(readstream($f,8)!=chr(137).'PNG'.chr(13).chr(10).chr(26).chr(10))
		echo('Not a PNG file: '.$file);
	// Read header chunk
	readstream($f,4);
	if(readstream($f,4)!='IHDR')
		echo('Incorrect PNG file: '.$file);
	$w = readint($f);
	$h = readint($f);
	$bpc = ord(readstream($f,1));
	if($bpc>8)
		echo('16-bit depth not supported: '.$file);
	$ct = ord(readstream($f,1));
	if($ct==0 || $ct==4)
		$colspace = 'DeviceGray';
	elseif($ct==2 || $ct==6)
		$colspace = 'DeviceRGB';
	elseif($ct==3)
		$colspace = 'Indexed';
	else
		echo('Unknown color type: '.$file);
	if(ord(readstream($f,1))!=0)
		echo('Unknown compression method: '.$file);
	if(ord(readstream($f,1))!=0)
		echo('Unknown filter method: '.$file);
	if(ord(readstream($f,1))!=0)
		echo('Interlacing not supported: '.$file);
	readstream($f,4);
	// Scan chunks looking for palette, transparency and image data
	$pal = '';
	$trns = '';
	$data = '';
	do
	{
		$n = readint($f);
		$type = readstream($f,4);
		if($type=='PLTE')
		{
			// Read palette
			$pal = readstream($f,$n);
			readstream($f,4);
		}
		elseif($type=='tRNS')
		{
			// Read transparency info
			$t = readstream($f,$n);
			if($ct==0)
				$trns = array(ord(substr($t,1,1)));
			elseif($ct==2)
				$trns = array(ord(substr($t,1,1)), ord(substr($t,3,1)), ord(substr($t,5,1)));
			else
			{
				$pos = strpos($t,chr(0));
				if($pos!==false)
					$trns = array($pos);
			}
			readstream($f,4);
		}
		elseif($type=='IDAT')
		{
			// Read image data block
			$data .= readstream($f,$n);
			readstream($f,4);
		}
		elseif($type=='IEND')
			break;
		else
			readstream($f,$n+4);
	}
	while($n);
	if($colspace=='Indexed' && empty($pal))
		$this->Error('Missing palette in '.$file);
	if($ct>=4)
	{
		// Extract alpha channel
		if(!function_exists('gzuncompress'))
			$this->Error('Zlib not available, can\'t handle alpha channel: '.$file);
		$data = gzuncompress($data);
		$color = '';
		$alpha = '';
		$lines = '';
		if($ct==4)
		{
			// Gray image
			$len = 2*$w;
			for($i=0;$i<$h;$i++)
			{
				$pos = (1+$len)*$i;
				$color .= $data[$pos];
				$alpha .= $data[$pos];
				$line = substr($data,$pos+1,$len);
				$color .= preg_replace('/(.)./s','$1',$line);
				$alpha .= preg_replace('/.(.)/s','$1',$line);
			}
		}
		else
		{
			// RGB image
			$len = 4*$w;
			for($i=0;$i<$h;$i++)
			{
				$pos = (1+$len)*$i;
				$color .= $data[$pos];
				$alpha .= $data[$pos];
				$line = substr($data,$pos+1,$len);
				$lines .= $line;
				$color .= preg_replace('/(.{3})./s','$1',$line);
				$alpha .= preg_replace('/.{3}(.)/s','$1',$line);
			}
		}
		unset($data);
	}
	file_put_contents('COLOR.TXT', $color);
	file_put_contents('ALPHA.TXT', $alpha);
}

function readstream($f, $n)
{
	// Read n bytes from stream
	$res = '';
	while($n>0 && !feof($f))
	{
		$s = fread($f,$n);
		if($s===false)
			echo('Error while reading stream');
		$n -= strlen($s);
		$res .= $s;
	}
	if($n>0)
		echo('Unexpected end of stream');
	return $res;
}

function readint($f)
{
	// Read a 4-byte integer from stream
	$a = unpack('Ni', readstream($f,4));
	return $a['i'];
}
parsepng('image.png');

?>